@@ -338,7 +338,7 @@ static int fontTest (void *element, void *key)
 {
 	font_t *a = (font_t *) element;
 	fontkey_t *b = (fontkey_t *) key;
-	
+
 	if (strcmp (a->fontlist, b->fontlist) == 0) {
 		switch (b->preferred_map) {
 			case gdFTEX_Unicode:
@@ -412,10 +412,9 @@ static void *fontFetch (char **error, void *key)
 		for (dir = gd_strtok_r (path, PATHSEPARATOR, &strtok_ptr_path); dir;
 		     dir = gd_strtok_r (0, PATHSEPARATOR, &strtok_ptr_path)) {
 			if (!strcmp(dir, ".")) {
-				TSRMLS_FETCH();
-#if HAVE_GETCWD
+			#ifdef HAVE_GETCWD
 				dir = VCWD_GETCWD(cur_dir, MAXPATHLEN);
-#elif HAVE_GETWD
+#elif defined(HAVE_GETWD)
 				dir = VCWD_GETWD(cur_dir);
 #endif
 				if (!dir) {
@@ -627,6 +626,7 @@ static char * gdft_draw_bitmap (gdCache_head_t *tc_cache, gdImage * im, int fg,
 {
 	unsigned char *pixel = NULL;
 	int *tpixel = NULL;
+	int opixel;
 	int x, y, row, col, pc, pcr;
 
 	tweencolor_t *tc_elem;
@@ -684,7 +684,13 @@ static char * gdft_draw_bitmap (gdCache_head_t *tc_cache, gdImage * im, int fg,
 					}
 				} else {
 					if (im->alphaBlendingFlag) {
-						*tpixel = gdAlphaBlend(*tpixel, (level << 24) + (fg & 0xFFFFFF));
+						opixel = *tpixel;
+						if (gdTrueColorGetAlpha(opixel) != gdAlphaTransparent) {
+							*tpixel = gdAlphaBlend (opixel,
+							                        (level << 24) + (fg & 0xFFFFFF));
+						} else {
+							*tpixel = (level << 24) + (fg & 0xFFFFFF);
+						}
 					} else {
 						*tpixel = (level << 24) + (fg & 0xFFFFFF);
 					}
