@@ -1,5 +1,3 @@
-
-	/* $Id: fpm_stdio.c,v 1.22.2.2 2008/12/13 03:32:24 anight Exp $ */
 	/* (c) 2007,2008 Andrei Nigmatulin */
 
 #include "fpm_config.h"
@@ -15,6 +13,7 @@
 
 #include "fpm.h"
 #include "fpm_children.h"
+#include "fpm_cleanup.h"
 #include "fpm_events.h"
 #include "fpm_sockets.h"
 #include "fpm_stdio.h"
@@ -50,7 +49,7 @@ static inline int fpm_use_error_log() {  /* {{{ */
 	 * - SysV init launch php-fpm as a daemon
 	 * - Systemd launch php-fpm in foreground
 	 */
-#if HAVE_UNISTD_H
+#ifdef HAVE_UNISTD_H
 	if (fpm_global_config.daemonize || (!isatty(STDERR_FILENO) && !fpm_globals.force_stderr)) {
 #else
 	if (fpm_global_config.daemonize) {
@@ -89,7 +88,7 @@ int fpm_stdio_init_child(struct fpm_worker_pool_s *wp) /* {{{ */
 {
 #ifdef HAVE_SYSLOG_H
 	if (fpm_globals.error_log_fd == ZLOG_SYSLOG) {
-		closelog(); /* ensure to close syslog not to interrupt with PHP syslog code */
+		php_closelog(); /* ensure to close syslog not to interrupt with PHP syslog code */
 	} else
 #endif
 
@@ -103,16 +102,18 @@ int fpm_stdio_init_child(struct fpm_worker_pool_s *wp) /* {{{ */
 	fpm_globals.error_log_fd = -1;
 	zlog_set_fd(-1);
 
-	if (wp->listening_socket != STDIN_FILENO) {
-		if (0 > dup2(wp->listening_socket, STDIN_FILENO)) {
-			zlog(ZLOG_SYSERROR, "failed to init child stdio: dup2()");
-			return -1;
-		}
-	}
 	return 0;
 }
 /* }}} */
 
+#define FPM_STDIO_CMD_FLUSH "\0fscf"
+
+int fpm_stdio_flush_child() /* {{{ */
+{
+	return write(STDERR_FILENO, FPM_STDIO_CMD_FLUSH, sizeof(FPM_STDIO_CMD_FLUSH));
+}
+/* }}} */
+
 static void fpm_stdio_child_said(struct fpm_event_s *ev, short which, void *arg) /* {{{ */
 {
 	static const int max_buf_size = 1024;
@@ -121,15 +122,15 @@ static void fpm_stdio_child_said(struct fpm_event_s *ev, short which, void *arg)
 	struct fpm_child_s *child;
 	int is_stdout;
 	struct fpm_event_s *event;
-	int fifo_in = 1, fifo_out = 1;
-	int is_last_message = 0;
-	int in_buf = 0;
-	int res;
+	int in_buf = 0, cmd_pos = 0, pos, start;
+	int read_fail = 0, create_log_stream;
+	struct zlog_stream *log_stream;
 
 	if (!arg) {
 		return;
 	}
 	child = (struct fpm_child_s *)arg;
+
 	is_stdout = (fd == child->fd_stdout);
 	if (is_stdout) {
 		event = &child->ev_stdout;
@@ -137,74 +138,96 @@ static void fpm_stdio_child_said(struct fpm_event_s *ev, short which, void *arg)
 		event = &child->ev_stderr;
 	}
 
-	while (fifo_in || fifo_out) {
-		if (fifo_in) {
-			res = read(fd, buf + in_buf, max_buf_size - 1 - in_buf);
-			if (res <= 0) { /* no data */
-				fifo_in = 0;
-				if (res < 0 && (errno == EAGAIN || errno == EWOULDBLOCK)) {
-					/* just no more data ready */
-				} else { /* error or pipe is closed */
+	create_log_stream = !child->log_stream;
+	if (create_log_stream) {
+		log_stream = child->log_stream = malloc(sizeof(struct zlog_stream));
+		zlog_stream_init_ex(log_stream, ZLOG_WARNING, STDERR_FILENO);
+		zlog_stream_set_decorating(log_stream, child->wp->config->decorate_workers_output);
+		zlog_stream_set_wrapping(log_stream, ZLOG_TRUE);
+		zlog_stream_set_msg_prefix(log_stream, STREAM_SET_MSG_PREFIX_FMT,
+				child->wp->config->name, (int) child->pid, is_stdout ? "stdout" : "stderr");
+		zlog_stream_set_msg_quoting(log_stream, ZLOG_TRUE);
+		zlog_stream_set_is_stdout(log_stream, is_stdout);
+		zlog_stream_set_child_pid(log_stream, (int)child->pid);
+	} else {
+		log_stream = child->log_stream;
+		// if fd type (stdout/stderr) or child's pid is changed,
+		// then the stream will be finished and msg's prefix will be reinitialized
+		if (log_stream->is_stdout != (unsigned int)is_stdout || log_stream->child_pid != (int)child->pid) {
+			zlog_stream_finish(log_stream);
+			zlog_stream_set_msg_prefix(log_stream, STREAM_SET_MSG_PREFIX_FMT,
+					child->wp->config->name, (int) child->pid, is_stdout ? "stdout" : "stderr");
+			zlog_stream_set_is_stdout(log_stream, is_stdout);
+			zlog_stream_set_child_pid(log_stream, (int)child->pid);
+		}
+	}
 
-					if (res < 0) { /* error */
-						zlog(ZLOG_SYSERROR, "unable to read what child say");
-					}
-
-					fpm_event_del(event);
-					is_last_message = 1;
-
-					if (is_stdout) {
-						close(child->fd_stdout);
-						child->fd_stdout = -1;
-					} else {
-						close(child->fd_stderr);
-						child->fd_stderr = -1;
-					}
-				}
+	while (1) {
+stdio_read:
+		in_buf = read(fd, buf, max_buf_size - 1);
+		if (in_buf <= 0) { /* no data */
+			if (in_buf == 0 || (errno != EAGAIN && errno != EWOULDBLOCK)) {
+				/* pipe is closed or error */
+				read_fail = (in_buf < 0) ? in_buf : 1;
+			}
+			break;
+		}
+		start = 0;
+		if (cmd_pos > 0) {
+			if 	((sizeof(FPM_STDIO_CMD_FLUSH) - cmd_pos) <= in_buf &&
+					!memcmp(buf, &FPM_STDIO_CMD_FLUSH[cmd_pos], sizeof(FPM_STDIO_CMD_FLUSH) - cmd_pos)) {
+				zlog_stream_finish(log_stream);
+				start = cmd_pos;
 			} else {
-				in_buf += res;
+				zlog_stream_str(log_stream, &FPM_STDIO_CMD_FLUSH[0], cmd_pos);
+			}
+			cmd_pos = 0;
+		}
+		for (pos = start; pos < in_buf; pos++) {
+			switch (buf[pos]) {
+				case '\n':
+					zlog_stream_str(log_stream, buf + start, pos - start);
+					zlog_stream_finish(log_stream);
+					start = pos + 1;
+					break;
+				case '\0':
+					if (pos + sizeof(FPM_STDIO_CMD_FLUSH) <= in_buf) {
+						if (!memcmp(buf + pos, FPM_STDIO_CMD_FLUSH, sizeof(FPM_STDIO_CMD_FLUSH))) {
+							zlog_stream_str(log_stream, buf + start, pos - start);
+							zlog_stream_finish(log_stream);
+							start = pos + sizeof(FPM_STDIO_CMD_FLUSH);
+							pos = start - 1;
+						}
+					} else if (!memcmp(buf + pos, FPM_STDIO_CMD_FLUSH, in_buf - pos)) {
+						cmd_pos = in_buf - pos;
+						zlog_stream_str(log_stream, buf + start, pos - start);
+						goto stdio_read;
+					}
+					break;
 			}
 		}
+		if (start < pos) {
+			zlog_stream_str(log_stream, buf + start, pos - start);
+		}
+	}
 
-		if (fifo_out) {
-			if (in_buf == 0) {
-				fifo_out = 0;
-			} else {
-				char *nl;
-				int should_print = 0;
-				buf[in_buf] = '\0';
+	if (read_fail) {
+		if (create_log_stream) {
+			zlog_stream_set_msg_suffix(log_stream, NULL, ", pipe is closed");
+			zlog_stream_finish(log_stream);
+		}
+		if (read_fail < 0) {
+			zlog(ZLOG_SYSERROR, "unable to read what child say");
+		}
 
-				/* FIXME: there might be binary data */
+		fpm_event_del(event);
 
-				/* we should print if no more space in the buffer */
-				if (in_buf == max_buf_size - 1) {
-					should_print = 1;
-				}
-
-				/* we should print if no more data to come */
-				if (!fifo_in) {
-					should_print = 1;
-				}
-
-				nl = strchr(buf, '\n');
-				if (nl || should_print) {
-
-					if (nl) {
-						*nl = '\0';
-					}
-
-					zlog(ZLOG_WARNING, "[pool %s] child %d said into %s: \"%s\"%s", child->wp->config->name,
-					  (int) child->pid, is_stdout ? "stdout" : "stderr", buf, is_last_message ? ", pipe is closed" : "");
-
-					if (nl) {
-						int out_buf = 1 + nl - buf;
-						memmove(buf, buf + out_buf, in_buf - out_buf);
-						in_buf -= out_buf;
-					} else {
-						in_buf = 0;
-					}
-				}
-			}
+		if (is_stdout) {
+			close(child->fd_stdout);
+			child->fd_stdout = -1;
+		} else {
+			close(child->fd_stderr);
+			child->fd_stderr = -1;
 		}
 	}
 }
@@ -296,7 +319,7 @@ int fpm_stdio_open_error_log(int reopen) /* {{{ */
 
 #ifdef HAVE_SYSLOG_H
 	if (!strcasecmp(fpm_global_config.error_log, "syslog")) {
-		openlog(fpm_global_config.syslog_ident, LOG_PID | LOG_CONS, fpm_global_config.syslog_facility);
+		php_openlog(fpm_global_config.syslog_ident, LOG_PID | LOG_CONS, fpm_global_config.syslog_facility);
 		fpm_globals.error_log_fd = ZLOG_SYSLOG;
 		if (fpm_use_error_log()) {
 			zlog_set_fd(fpm_globals.error_log_fd);
@@ -331,4 +354,3 @@ int fpm_stdio_open_error_log(int reopen) /* {{{ */
 	return 0;
 }
 /* }}} */
-
